@@ -24,6 +24,10 @@
             Darin Adler <darin@bentspoon.com>,
 	    Andy Hertzfeld <andy@eazel.com>
 	    Pavel Cisler <pavel@eazel.com>
+	    
+
+   XDS support: Benedikt Meurer <benny@xfce.org> (adapted by Amos Brocco <amos.brocco@unifr.ch>)
+				
 */
 
 
@@ -31,6 +35,7 @@
 #include <math.h>
 #include "nautilus-icon-dnd.h"
 
+#include "nautilus-debug-log.h"
 #include "nautilus-file-dnd.h"
 #include "nautilus-icon-private.h"
 #include "nautilus-link.h"
@@ -54,27 +59,26 @@
 #include <eel/eel-canvas-rect-ellipse.h>
 #include <libgnomeui/gnome-stock-icons.h>
 #include <libgnomeui/gnome-uidefs.h>
-#include <libgnomevfs/gnome-vfs-uri.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
-#include <libgnomevfs/gnome-vfs-mime-utils.h>
 #include <libnautilus-private/nautilus-file-utilities.h>
+#include <libnautilus-private/nautilus-file-changes-queue.h>
 #include <stdio.h>
 #include <string.h>
 
 static const GtkTargetEntry drag_types [] = {
 	{ NAUTILUS_ICON_DND_GNOME_ICON_LIST_TYPE, 0, NAUTILUS_ICON_DND_GNOME_ICON_LIST },
 	{ NAUTILUS_ICON_DND_URI_LIST_TYPE, 0, NAUTILUS_ICON_DND_URI_LIST },
-	{ NAUTILUS_ICON_DND_URL_TYPE, 0, NAUTILUS_ICON_DND_URL },
 };
 
 static const GtkTargetEntry drop_types [] = {
 	{ NAUTILUS_ICON_DND_GNOME_ICON_LIST_TYPE, 0, NAUTILUS_ICON_DND_GNOME_ICON_LIST },
+	/* prefer "_NETSCAPE_URL" over "text/uri-list" to satisfy web browsers. */
+	{ NAUTILUS_ICON_DND_NETSCAPE_URL_TYPE, 0, NAUTILUS_ICON_DND_NETSCAPE_URL },
 	{ NAUTILUS_ICON_DND_URI_LIST_TYPE, 0, NAUTILUS_ICON_DND_URI_LIST },
-	{ NAUTILUS_ICON_DND_URL_TYPE, 0, NAUTILUS_ICON_DND_URL },
 	{ NAUTILUS_ICON_DND_COLOR_TYPE, 0, NAUTILUS_ICON_DND_COLOR },
 	{ NAUTILUS_ICON_DND_BGIMAGE_TYPE, 0, NAUTILUS_ICON_DND_BGIMAGE },
 	{ NAUTILUS_ICON_DND_KEYWORD_TYPE, 0, NAUTILUS_ICON_DND_KEYWORD },
 	{ NAUTILUS_ICON_DND_RESET_BACKGROUND_TYPE,  0, NAUTILUS_ICON_DND_RESET_BACKGROUND },
+	{ NAUTILUS_ICON_DND_XDNDDIRECTSAVE_TYPE, 0, NAUTILUS_ICON_DND_XDNDDIRECTSAVE }, /* XDS Protocol Type */
 	/* Must be last: */
 	{ NAUTILUS_ICON_DND_ROOTWINDOW_DROP_TYPE,  0, NAUTILUS_ICON_DND_ROOTWINDOW_DROP }
 };
@@ -84,6 +88,11 @@ static void     dnd_highlight_queue_redraw (GtkWidget      *widget);
 static GtkTargetList *drop_types_list = NULL;
 static GtkTargetList *drop_types_list_root = NULL;
 
+static char * nautilus_icon_container_find_drop_target (NautilusIconContainer *container,
+							GdkDragContext *context,
+							int x, int y, gboolean *icon_hit,
+							gboolean rewrite_desktop);
+
 static EelCanvasItem *
 create_selection_shadow (NautilusIconContainer *container,
 			 GList *list)
@@ -182,10 +191,10 @@ typedef struct {
 
 static void
 canvas_rect_world_to_widget (EelCanvas *canvas,
-			     ArtDRect *world_rect,
-			     ArtIRect *widget_rect)
+			     EelDRect *world_rect,
+			     EelIRect *widget_rect)
 {
-	ArtDRect window_rect;
+	EelDRect window_rect;
 	
 	eel_canvas_world_to_window (canvas,
 				    world_rect->x0, world_rect->y0,
@@ -214,8 +223,8 @@ static gboolean
 icon_get_data_binder (NautilusIcon *icon, gpointer data)
 {
 	IconGetDataBinderContext *context;
-	ArtDRect world_rect;
-	ArtIRect widget_rect;
+	EelDRect world_rect;
+	EelIRect widget_rect;
 	char *uri;
 	NautilusIconContainer *container;
 
@@ -235,11 +244,11 @@ icon_get_data_binder (NautilusIcon *icon, gpointer data)
 		return TRUE;
 	}
 
-	widget_rect = eel_art_irect_offset_by (widget_rect, 
+	widget_rect = eel_irect_offset_by (widget_rect, 
 		- container->details->dnd_info->drag_info.start_x,
 		- container->details->dnd_info->drag_info.start_y);
 
-	widget_rect = eel_art_irect_scale_by (widget_rect, 
+	widget_rect = eel_irect_scale_by (widget_rect, 
 		1 / EEL_CANVAS (container)->pixels_per_unit);
 	
 	/* pass the uri, mouse-relative x/y and icon width/height */
@@ -367,13 +376,84 @@ nautilus_icon_container_dropped_icon_feedback (GtkWidget *widget,
 	nautilus_icon_container_position_shadow (container, x, y);
 }
 
+static char *
+get_direct_save_filename (GdkDragContext *context)
+{
+	guchar *prop_text;
+	gint prop_len;
+	
+	if (!gdk_property_get (context->source_window, gdk_atom_intern (NAUTILUS_ICON_DND_XDNDDIRECTSAVE_TYPE, FALSE),
+			       gdk_atom_intern ("text/plain", FALSE), 0, 1024, FALSE, NULL, NULL,
+			       &prop_len, &prop_text) && prop_text != NULL) {
+		return NULL;
+	}
+	
+	/* Zero-terminate the string */
+	prop_text = g_realloc (prop_text, prop_len + 1);
+	prop_text[prop_len] = '\0';
+	
+	/* Verify that the file name provided by the source is valid */
+	if (*prop_text == '\0' ||
+	    strchr ((const gchar *) prop_text, G_DIR_SEPARATOR) != NULL) {
+		nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
+				    "Invalid filename provided by XDS drag site");
+		g_free (prop_text);
+		return NULL;
+	}
+	
+	return prop_text;
+}
+
+static void
+set_direct_save_uri (GtkWidget *widget, GdkDragContext *context, NautilusDragInfo *drag_info, int x, int y)
+{
+	GFile *base, *child;
+	char *filename, *drop_target;
+	gchar *uri;
+	
+	drag_info->got_drop_data_type = TRUE;
+	drag_info->data_type = NAUTILUS_ICON_DND_XDNDDIRECTSAVE;
+	
+	uri = NULL;
+	
+	filename = get_direct_save_filename (context);
+	drop_target = nautilus_icon_container_find_drop_target (NAUTILUS_ICON_CONTAINER (widget), 
+								context, x, y, NULL, TRUE);
+	
+	if (drop_target && eel_uri_is_trash (drop_target)) {
+		g_free (drop_target);
+		drop_target = NULL; /* Cannot save to trash ...*/
+	}
+	
+	if (filename != NULL && drop_target != NULL) {
+		/* Resolve relative path */
+		base = g_file_new_for_uri (drop_target);
+		child = g_file_get_child (base, filename);
+		uri = g_file_get_uri (child);
+		g_object_unref (base);
+		g_object_unref (child);
+		
+		/* Change the uri property */
+		gdk_property_change (GDK_DRAWABLE (context->source_window),
+				     gdk_atom_intern (NAUTILUS_ICON_DND_XDNDDIRECTSAVE_TYPE, FALSE),
+				     gdk_atom_intern ("text/plain", FALSE), 8,
+				     GDK_PROP_MODE_REPLACE, (const guchar *) uri,
+				     strlen (uri));
+		
+		drag_info->direct_save_uri = uri;
+	} 
+	
+	g_free (filename);
+	g_free (drop_target);
+}
+
 /* FIXME bugzilla.gnome.org 47445: Needs to become a shared function */
 static void
-get_data_on_first_target_we_support (GtkWidget *widget, GdkDragContext *context, guint32 time)
+get_data_on_first_target_we_support (GtkWidget *widget, GdkDragContext *context, guint32 time, int x, int y)
 {
 	GtkTargetList *list;
 	GdkAtom target;
-	
+
 	if (drop_types_list == NULL) {
 		drop_types_list = gtk_target_list_new (drop_types,
 						       G_N_ELEMENTS (drop_types) - 1);
@@ -384,7 +464,7 @@ get_data_on_first_target_we_support (GtkWidget *widget, GdkDragContext *context,
 							    G_N_ELEMENTS (drop_types));
 		gtk_target_list_add_text_targets (drop_types_list_root, NAUTILUS_ICON_DND_TEXT);
 	}
-
+	
 	if (nautilus_icon_container_get_is_desktop (NAUTILUS_ICON_CONTAINER (widget))) {
 		list = drop_types_list_root;
 	} else {
@@ -402,14 +482,18 @@ get_data_on_first_target_we_support (GtkWidget *widget, GdkDragContext *context,
 		found = gtk_target_list_find (list, target, &info);
 		g_assert (found);
 
-		/* Don't get_data for rootwindow drops unless it's the actual drop */
-		if (info == NAUTILUS_ICON_DND_ROOTWINDOW_DROP &&
+		/* Don't get_data for destructive ops */
+		if ((info == NAUTILUS_ICON_DND_ROOTWINDOW_DROP ||
+		     info == NAUTILUS_ICON_DND_XDNDDIRECTSAVE) &&
 		    !drag_info->drop_occured) {
 			/* We can't call get_data here, because that would
-			   make the source execute the rootwin action */
+			   make the source execute the rootwin action or the direct save */
 			drag_info->got_drop_data_type = TRUE;
-			drag_info->data_type = NAUTILUS_ICON_DND_ROOTWINDOW_DROP;
+			drag_info->data_type = info;
 		} else {
+			if (info == NAUTILUS_ICON_DND_XDNDDIRECTSAVE) {
+				set_direct_save_uri (widget, context, drag_info, x, y);
+			}
 			gtk_drag_get_data (GTK_WIDGET (widget), context,
 					   target, time);
 		}
@@ -426,7 +510,7 @@ nautilus_icon_container_ensure_drag_data (NautilusIconContainer *container,
 	dnd_info = container->details->dnd_info;
 
 	if (!dnd_info->drag_info.got_drop_data_type) {
-		get_data_on_first_target_we_support (GTK_WIDGET (container), context, time);
+		get_data_on_first_target_we_support (GTK_WIDGET (container), context, time, 0, 0);
 	}
 }
 
@@ -451,8 +535,8 @@ nautilus_icon_container_item_at (NautilusIconContainer *container,
 {
 	GList *p;
 	int size;
-	ArtDRect point;
-	ArtIRect canvas_point;
+	EelDRect point;
+	EelIRect canvas_point;
 
 	/* build the hit-test rectangle. Base the size on the scale factor to ensure that it is
 	 * non-empty even at the smallest scale factor
@@ -512,12 +596,7 @@ nautilus_icon_container_selection_items_local (NautilusIconContainer *container,
 	/* get the URI associated with the container */
 	container_uri_string = get_container_uri (container);
 	
-	if (eel_uri_is_trash (container_uri_string)) {
-		/* Special-case "trash:" because the nautilus_drag_items_local
-		 * would not work for it.
-		 */
-		result = nautilus_drag_items_in_trash (items);
-	} else if (eel_uri_is_desktop (container_uri_string)) {
+	if (eel_uri_is_desktop (container_uri_string)) {
 		result = nautilus_drag_items_on_desktop (items);
 	} else {
 		result = nautilus_drag_items_local (container_uri_string, items);
@@ -555,8 +634,15 @@ receive_dropped_color (NautilusIconContainer *container,
 		       GtkSelectionData *data)
 {
 	action = get_background_drag_action (container, action);
-	
+
 	if (action > 0) {
+		char *uri;
+
+		uri = get_container_uri (container);
+		nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
+				    "dropped color on icon container displaying %s", uri);
+		g_free (uri);
+
 		eel_background_receive_dropped_color
 			(eel_get_widget_background (GTK_WIDGET (container)),
 			 GTK_WIDGET (container), 
@@ -573,6 +659,13 @@ receive_dropped_tile_image (NautilusIconContainer *container, GdkDragAction acti
 	action = get_background_drag_action (container, action);
 
 	if (action > 0) {
+		char *uri;
+
+		uri = get_container_uri (container);
+		nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
+				    "dropped tile image on icon container displaying %s", uri);
+		g_free (uri);
+
 		eel_background_receive_dropped_background_image
 			(eel_get_widget_background (GTK_WIDGET (container)), 
 			 action, 
@@ -607,7 +700,12 @@ receive_dropped_keyword (NautilusIconContainer *container, const char *keyword,
 	 * so we don't have to worry about async. issues here.
 	 */
 	uri = nautilus_icon_container_get_icon_uri (container, drop_target_icon);
-	file = nautilus_file_get (uri);
+
+	nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
+			    "dropped emblem '%s' on icon container URI: %s",
+			    keyword, uri);
+
+	file = nautilus_file_get_by_uri (uri);
 	g_free (uri);
 	
 	nautilus_drag_file_receive_dropped_keyword (file, keyword);
@@ -618,44 +716,65 @@ receive_dropped_keyword (NautilusIconContainer *container, const char *keyword,
 
 /* handle dropped url */
 static void
-receive_dropped_url (NautilusIconContainer *container, const char *encoded_url, GdkDragAction action, int x, int y)
+receive_dropped_netscape_url (NautilusIconContainer *container, const char *encoded_url, GdkDragContext *context, int x, int y)
 {
+	char *drop_target;
+
 	if (encoded_url == NULL) {
 		return;
 	}
 
-	g_signal_emit_by_name (container, "handle_url",
+	drop_target = nautilus_icon_container_find_drop_target (container, context, x, y, NULL, TRUE);
+
+	g_signal_emit_by_name (container, "handle_netscape_url",
 			       encoded_url,
-			       action,
+			       drop_target,
+			       context->action,
 			       x, y);
+
+	g_free (drop_target);
 }
 
 /* handle dropped uri list */
 static void
-receive_dropped_uri_list (NautilusIconContainer *container, const char *uri_list, GdkDragAction action, int x, int y)
+receive_dropped_uri_list (NautilusIconContainer *container, const char *uri_list, GdkDragContext *context, int x, int y)
 {	
+	char *drop_target;
+
 	if (uri_list == NULL) {
 		return;
 	}
-	
+
+	drop_target = nautilus_icon_container_find_drop_target (container, context, x, y, NULL, TRUE);
+
 	g_signal_emit_by_name (container, "handle_uri_list",
 				 uri_list,
-				 action,
+				 drop_target,
+				 context->action,
 				 x, y);
+
+	g_free (drop_target);
 }
 
 /* handle dropped text */
 static void
-receive_dropped_text (NautilusIconContainer *container, const char *text, GdkDragAction action, int x, int y)
+receive_dropped_text (NautilusIconContainer *container, const char *text, GdkDragContext *context, int x, int y)
 {	
+	char *drop_target;
+
 	if (text == NULL) {
 		return;
 	}
+
+	drop_target = nautilus_icon_container_find_drop_target (container, context, x, y, NULL, TRUE);
 	
 	g_signal_emit_by_name (container, "handle_text",
 			       text,
-			       action,
+			       drop_target,
+			       context->action,
 			       x, y);
+
+	g_free (drop_target);
 }
 
 static int
@@ -806,7 +925,7 @@ handle_local_move (NautilusIconContainer *container,
 	NautilusDragSelectionItem *item;
 	NautilusIcon *icon;
 	NautilusFile *file;
-	char *screen_string;
+	char screen_string[32];
 	GdkScreen *screen;
 
 	if (container->details->auto_layout) {
@@ -830,17 +949,15 @@ handle_local_move (NautilusIconContainer *container,
 			 * this screen
 			 */
 
-			file = nautilus_file_get (item->uri);
+			file = nautilus_file_get_by_uri (item->uri);
 
 			screen = gtk_widget_get_screen (GTK_WIDGET (container));
-			screen_string = g_strdup_printf ("%d",
-						gdk_screen_get_number (screen));
+			g_snprintf (screen_string, sizeof (screen_string), "%d",
+				    gdk_screen_get_number (screen));
 			nautilus_file_set_metadata (file,
 					NAUTILUS_METADATA_KEY_SCREEN,
 					NULL, screen_string);
 
-			g_free (screen_string);
-
 			nautilus_icon_container_add (container,
 					NAUTILUS_ICON_CONTAINER_ICON_DATA (file),
 					FALSE);
@@ -874,8 +991,8 @@ handle_nonlocal_move (NautilusIconContainer *container,
 {
 	GList *source_uris, *p;
 	GArray *source_item_locations;
-	gboolean free_target_uri;
-	int index;
+	gboolean free_target_uri, is_rtl;
+	int index, item_x;
 
 	if (container->details->dnd_info->drag_info.selection_list == NULL) {
 		return;
@@ -888,6 +1005,8 @@ handle_nonlocal_move (NautilusIconContainer *container,
 	}
 	source_uris = g_list_reverse (source_uris);
 	
+	is_rtl = nautilus_icon_container_is_layout_rtl (container);
+
 	source_item_locations = g_array_new (FALSE, TRUE, sizeof (GdkPoint));
 	if (!icon_hit) {
 		/* Drop onto a container. Pass along the item points to allow placing
@@ -898,8 +1017,10 @@ handle_nonlocal_move (NautilusIconContainer *container,
 			
 		for (index = 0, p = container->details->dnd_info->drag_info.selection_list;
 			p != NULL; index++, p = p->next) {
-		     	g_array_index (source_item_locations, GdkPoint, index).x =
-		     		((NautilusDragSelectionItem *)p->data)->icon_x;
+		    	item_x = ((NautilusDragSelectionItem *)p->data)->icon_x;
+			if (is_rtl)
+				item_x = -item_x - ((NautilusDragSelectionItem *)p->data)->icon_width;
+		     	g_array_index (source_item_locations, GdkPoint, index).x = item_x;
 		     	g_array_index (source_item_locations, GdkPoint, index).y =
 				((NautilusDragSelectionItem *)p->data)->icon_y;
 		}
@@ -911,7 +1032,11 @@ handle_nonlocal_move (NautilusIconContainer *container,
 		target_uri = nautilus_get_desktop_directory_uri ();
 		free_target_uri = TRUE;
 	}
-	
+
+	if (is_rtl) {
+		x = CANVAS_WIDTH (container) - x;
+	}
+
 	/* start the copy */
 	g_signal_emit_by_name (container, "move_copy_items",
 				 source_uris,
@@ -932,15 +1057,20 @@ static char *
 nautilus_icon_container_find_drop_target (NautilusIconContainer *container,
 					  GdkDragContext *context,
 					  int x, int y,
-					  gboolean *icon_hit)
+					  gboolean *icon_hit,
+					  gboolean rewrite_desktop)
 {
 	NautilusIcon *drop_target_icon;
 	double world_x, world_y;
 	NautilusFile *file;
 	char *icon_uri;
+	char *container_uri;
+	
+	if (icon_hit) {
+		*icon_hit = FALSE;
+	}
 
-	*icon_hit = FALSE;
-	if (container->details->dnd_info->drag_info.selection_list == NULL) {
+	if (!container->details->dnd_info->drag_info.got_drop_data_type) {
 		return NULL;
 	}
 
@@ -957,10 +1087,11 @@ nautilus_icon_container_find_drop_target (NautilusIconContainer *container,
 	if (drop_target_icon != NULL) {
 		icon_uri = nautilus_icon_container_get_icon_uri (container, drop_target_icon);
 		if (icon_uri != NULL) {
-			file = nautilus_file_get (icon_uri);
+			file = nautilus_file_get_by_uri (icon_uri);
 
-			if (!nautilus_drag_can_accept_items (file, 
-					container->details->dnd_info->drag_info.selection_list)) {
+			if (!nautilus_drag_can_accept_info (file,
+							    container->details->dnd_info->drag_info.data_type,
+							    container->details->dnd_info->drag_info.selection_list)) {
 			 	/* the item we dropped our selection on cannot accept the items,
 			 	 * do the same thing as if we just dropped the items on the canvas
 				 */
@@ -973,20 +1104,36 @@ nautilus_icon_container_find_drop_target (NautilusIconContainer *container,
 	}
 
 	if (drop_target_icon == NULL) {
-		*icon_hit = FALSE;
-		return get_container_uri (container);
+		if (icon_hit) {
+			*icon_hit = FALSE;
+		}
+
+		container_uri = get_container_uri (container);
+
+		if (rewrite_desktop &&
+		    container_uri != NULL &&
+		    eel_uri_is_desktop (container_uri)) {
+			g_free (container_uri);
+			container_uri = nautilus_get_desktop_directory_uri ();
+		}
+		
+		return container_uri;
 	}
 	
-	*icon_hit = TRUE;
+	if (icon_hit) {
+		*icon_hit = TRUE;
+	}
 	return nautilus_icon_container_get_icon_drop_target_uri (container, drop_target_icon);
 }
 
 static gboolean
 selection_is_image_file (GList *selection_list)
 {
-	char *mime_type;
+	const char *mime_type;
 	NautilusDragSelectionItem *selected_item;
 	gboolean result;
+	GFile *location;
+	GFileInfo *info;
 
 	/* Make sure only one item is selected */
 	if (selection_list == NULL ||
@@ -996,11 +1143,22 @@ selection_is_image_file (GList *selection_list)
 
 	selected_item = selection_list->data;
 
-	mime_type = gnome_vfs_get_mime_type (selected_item->uri);
+	mime_type = NULL;
+	
+	location = g_file_new_for_uri (selected_item->uri);
+	info = g_file_query_info (location,
+				  G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
+				  0, NULL, NULL);
+	if (info) {
+		mime_type = g_file_info_get_content_type (info);
+	}
 
 	result = eel_istr_has_prefix (mime_type, "image/");
-	
-	g_free (mime_type);
+
+	if (info) {
+		g_object_unref (info);
+	}
+	g_object_unref (location);
 	
 	return result;
 }
@@ -1057,7 +1215,7 @@ nautilus_icon_container_receive_dropped_icons (NautilusIconContainer *container,
 					    &world_x, &world_y);
 
 		drop_target = nautilus_icon_container_find_drop_target (container, 
-			context, x, y, &icon_hit);
+									context, x, y, &icon_hit, FALSE);
 
 		local_move_only = FALSE;
 		if (!icon_hit && context->action == GDK_ACTION_MOVE) {
@@ -1111,13 +1269,20 @@ nautilus_icon_container_get_drop_action (NautilusIconContainer *container,
 			return;
 		}
 		drop_target = nautilus_icon_container_find_drop_target (container,
-			context, x, y, &icon_hit);
+									context, x, y, &icon_hit, FALSE);
 		if (!drop_target) {
 			return;
 		}
 		nautilus_drag_default_drop_action_for_icons (context, drop_target, 
-			container->details->dnd_info->drag_info.selection_list, 
-			action);
+							     container->details->dnd_info->drag_info.selection_list, 
+							     action);
+		g_free (drop_target);
+		break;
+	case NAUTILUS_ICON_DND_URI_LIST:
+		drop_target = nautilus_icon_container_find_drop_target (container,
+									context, x, y, &icon_hit, FALSE);
+		*action = nautilus_drag_default_drop_action_for_uri_list (context, drop_target);
+
 		g_free (drop_target);
 		break;
 
@@ -1128,19 +1293,19 @@ nautilus_icon_container_get_drop_action (NautilusIconContainer *container,
 		}
 		break;
 
-	case NAUTILUS_ICON_DND_URL:
-		*action = nautilus_drag_default_drop_action_for_url (context);
+	case NAUTILUS_ICON_DND_NETSCAPE_URL:
+		*action = nautilus_drag_default_drop_action_for_netscape_url (context);
 		break;
 
 	case NAUTILUS_ICON_DND_COLOR:
 	case NAUTILUS_ICON_DND_BGIMAGE:
 	case NAUTILUS_ICON_DND_RESET_BACKGROUND:
-	case NAUTILUS_ICON_DND_URI_LIST:
 	case NAUTILUS_ICON_DND_ROOTWINDOW_DROP:
 		*action = context->suggested_action;
 		break;
 
 	case NAUTILUS_ICON_DND_TEXT:
+	case NAUTILUS_ICON_DND_XDNDDIRECTSAVE:
 		*action = GDK_ACTION_COPY;
 		break;
 	}
@@ -1177,10 +1342,6 @@ nautilus_icon_dnd_update_drop_target (NautilusIconContainer *container,
 	char *uri;
 	
 	g_assert (NAUTILUS_IS_ICON_CONTAINER (container));
-	if ((container->details->dnd_info->drag_info.selection_list == NULL) 
-	   && (container->details->dnd_info->drag_info.data_type != NAUTILUS_ICON_DND_KEYWORD)) {
-		return;
-	}
 
 	canvas_widget_to_world (EEL_CANVAS (container), x, y, &world_x, &world_y);
 
@@ -1196,11 +1357,12 @@ nautilus_icon_dnd_update_drop_target (NautilusIconContainer *container,
 	/* Find if target icon accepts our drop. */
 	if (icon != NULL && (container->details->dnd_info->drag_info.data_type != NAUTILUS_ICON_DND_KEYWORD)) {
 		    uri = nautilus_icon_container_get_icon_uri (container, icon);
-		    file = nautilus_file_get (uri);
+		    file = nautilus_file_get_by_uri (uri);
 		    g_free (uri);
 		
-		    if (!nautilus_drag_can_accept_items (file,
-							 container->details->dnd_info->drag_info.selection_list)) {
+		    if (!nautilus_drag_can_accept_info (file,
+					    		container->details->dnd_info->drag_info.data_type,
+							container->details->dnd_info->drag_info.selection_list)) {
 			    icon = NULL;
 		    }
 
@@ -1214,7 +1376,7 @@ static void
 nautilus_icon_container_free_drag_data (NautilusIconContainer *container)
 {
 	NautilusIconDndInfo *dnd_info;
-	
+
 	dnd_info = container->details->dnd_info;
 	
 	dnd_info->drag_info.got_drop_data_type = FALSE;
@@ -1228,6 +1390,11 @@ nautilus_icon_container_free_drag_data (NautilusIconContainer *container)
 		gtk_selection_data_free (dnd_info->drag_info.selection_data);
 		dnd_info->drag_info.selection_data = NULL;
 	}
+
+	if (dnd_info->drag_info.direct_save_uri != NULL) {
+		g_free (dnd_info->drag_info.direct_save_uri);
+		dnd_info->drag_info.direct_save_uri = NULL;
+	}
 }
 
 static void
@@ -1256,19 +1423,37 @@ drag_begin_callback (GtkWidget      *widget,
 		     gpointer        data)
 {
 	NautilusIconContainer *container;
+	GdkScreen *screen;
+	GdkColormap *colormap;
 	GdkPixmap *pixmap;
 	GdkBitmap *mask;
 	double x1, y1, x2, y2, winx, winy;
 	int x_offset, y_offset;
 	int start_x, start_y;
+	gboolean use_mask;
 
 	container = NAUTILUS_ICON_CONTAINER (widget);
 
+	screen = gtk_widget_get_screen (widget);
+	colormap = NULL;
+	if (gdk_screen_is_composited (screen)) {
+		colormap = gdk_screen_get_rgba_colormap (screen);
+		if (colormap != NULL) {
+			use_mask = FALSE;
+		}
+	}
+	
+	/* Fall back on using the same colormap as the widget */
+	if (colormap == NULL) {
+		colormap = gtk_widget_get_colormap (widget);		
+		use_mask = TRUE;
+	}
+
 	start_x = container->details->dnd_info->drag_info.start_x + gtk_adjustment_get_value (gtk_layout_get_hadjustment (GTK_LAYOUT (container)));
 	start_y = container->details->dnd_info->drag_info.start_y + gtk_adjustment_get_value (gtk_layout_get_vadjustment (GTK_LAYOUT (container)));
 
         /* create a pixmap and mask to drag with */
-        pixmap = nautilus_icon_canvas_item_get_image (container->details->drag_icon->item, &mask);
+        pixmap = nautilus_icon_canvas_item_get_image (container->details->drag_icon->item, &mask, colormap);
     
     	/* we want to drag semi-transparent pixbufs, but X is too slow dealing with
 	   stippled masks, so we had to remove the code; this comment is left as a memorial
@@ -1282,9 +1467,20 @@ drag_begin_callback (GtkWidget      *widget,
         x_offset = start_x - winx;
         y_offset = start_y - winy;
 
+	if (!use_mask && pixmap != NULL) {
+		cairo_t *cr;
+
+		/* If composite works, make the icons partially transparent */
+		cr = gdk_cairo_create (pixmap);
+		cairo_set_operator (cr, CAIRO_OPERATOR_DEST_OUT);
+		cairo_set_source_rgba(cr, 1,0,0,0.35);
+		cairo_paint (cr);
+		cairo_destroy (cr);
+	}
+	
 	gtk_drag_set_icon_pixmap (context,
-				  gtk_widget_get_colormap (GTK_WIDGET (container)),
-				  pixmap, mask,
+				  colormap,
+				  pixmap, (use_mask ? mask : NULL),
 				  x_offset, y_offset);
 }
 
@@ -1425,7 +1621,7 @@ drag_motion_callback (GtkWidget *widget,
 		      guint32 time)
 {
 	int action;
-	
+
 	nautilus_icon_container_ensure_drag_data (NAUTILUS_ICON_CONTAINER (widget), context, time);
 	nautilus_icon_container_position_shadow (NAUTILUS_ICON_CONTAINER (widget), x, y);
 	nautilus_icon_dnd_update_drop_target (NAUTILUS_ICON_CONTAINER (widget), context, x, y);
@@ -1464,8 +1660,8 @@ drag_drop_callback (GtkWidget *widget,
 	*/
 	dnd_info->drag_info.drop_occured = TRUE;
 
-	get_data_on_first_target_we_support (widget, context, time);
-
+	get_data_on_first_target_we_support (widget, context, time, x, y);
+	
 	return FALSE;
 }
 
@@ -1484,74 +1680,6 @@ nautilus_icon_dnd_end_drag (NautilusIconContainer *container)
 	 */
 }
 
-/**
- * implements the gnome 1.x gnome_vfs_uri_list_extract_uris(), which
- * returns a GList of char *uris.
- **/
-GList *
-nautilus_icon_dnd_uri_list_extract_uris (const char *uri_list)
-{
-	/* Note that this is mostly very stolen from old libgnome/gnome-mime.c */
-
-	const gchar *p, *q;
-	gchar *retval;
-	GList *result = NULL;
-
-	g_return_val_if_fail (uri_list != NULL, NULL);
-
-	p = uri_list;
-
-	/* We don't actually try to validate the URI according to RFC
-	 * 2396, or even check for allowed characters - we just ignore
-	 * comments and trim whitespace off the ends.  We also
-	 * allow LF delimination as well as the specified CRLF.
-	 */
-	while (p != NULL) {
-		if (*p != '#') {
-			while (g_ascii_isspace (*p))
-				p++;
-
-			q = p;
-			while ((*q != '\0')
-			       && (*q != '\n')
-			       && (*q != '\r'))
-				q++;
-
-			if (q > p) {
-				q--;
-				while (q > p
-				       && g_ascii_isspace (*q))
-					q--;
-
-				retval = g_malloc (q - p + 2);
-				strncpy (retval, p, q - p + 1);
-				retval[q - p + 1] = '\0';
-
-				result = g_list_prepend (result, retval);
-			}
-		}
-		p = strchr (p, '\n');
-		if (p != NULL)
-			p++;
-	}
-
-	return g_list_reverse (result);
-}
-
-/**
- * nautilus_icon_dnd_uri_list_free_strings:
- * @list: A GList returned by nautilus_icon_dnd_uri_list_extract_uris()
- *
- * Releases all of the resources allocated by @list.
- * 
- * stolen from gnome-mime.c
- */
-void
-nautilus_icon_dnd_uri_list_free_strings (GList *list)
-{
-	eel_g_list_free_deep (list);
-}
-
 /** this callback is called in 2 cases.
     It is called upon drag_motion events to get the actual data 
     In that case, it just makes sure it gets the data.
@@ -1590,6 +1718,7 @@ drag_data_received_callback (GtkWidget *widget,
 	case NAUTILUS_ICON_DND_URI_LIST:
 	case NAUTILUS_ICON_DND_TEXT:
 	case NAUTILUS_ICON_DND_RESET_BACKGROUND:
+	case NAUTILUS_ICON_DND_XDNDDIRECTSAVE:
 		/* Save the data so we can do the actual work on drop. */
 		if (drag_info->selection_data != NULL) {
 			gtk_selection_data_free (drag_info->selection_data);
@@ -1598,7 +1727,7 @@ drag_data_received_callback (GtkWidget *widget,
 		break;
 
 	/* Netscape keeps sending us the data, even though we accept the first drag */
-	case NAUTILUS_ICON_DND_URL:
+	case NAUTILUS_ICON_DND_NETSCAPE_URL:
 		if (drag_info->selection_data != NULL) {
 			gtk_selection_data_free (drag_info->selection_data);
 			drag_info->selection_data = gtk_selection_data_copy (data);
@@ -1639,23 +1768,23 @@ drag_data_received_callback (GtkWidget *widget,
 				(NAUTILUS_ICON_CONTAINER (widget),
 				 (char *) data->data, x, y);
 			break;
-		case NAUTILUS_ICON_DND_URL:
-			receive_dropped_url
+		case NAUTILUS_ICON_DND_NETSCAPE_URL:
+			receive_dropped_netscape_url
 				(NAUTILUS_ICON_CONTAINER (widget),
-				 (char *) data->data, context->action, x, y);
+				 (char *) data->data, context, x, y);
 			success = TRUE;
 			break;
 		case NAUTILUS_ICON_DND_URI_LIST:
 			receive_dropped_uri_list
 				(NAUTILUS_ICON_CONTAINER (widget),
-				 (char *) data->data, context->action, x, y);
+				 (char *) data->data, context, x, y);
 			success = TRUE;
 			break;
 		case NAUTILUS_ICON_DND_TEXT:
 			tmp = gtk_selection_data_get_text (data);
 			receive_dropped_text
 				(NAUTILUS_ICON_CONTAINER (widget),
-				 (char *) tmp, context->action, x, y);
+				 (char *) tmp, context, x, y);
 			success = TRUE;
 			g_free (tmp);
 			break;
@@ -1669,6 +1798,34 @@ drag_data_received_callback (GtkWidget *widget,
 		case NAUTILUS_ICON_DND_ROOTWINDOW_DROP:
 			/* Do nothing, everything is done by the sender */
 			break;
+		case NAUTILUS_ICON_DND_XDNDDIRECTSAVE:
+			/* Indicate that we don't provide "F" fallback */
+          		if (drag_info->selection_data->format == 8 &&
+			    drag_info->selection_data->length == 1 &&
+			    drag_info->selection_data->data[0] == 'F') {
+	              		gdk_property_change (GDK_DRAWABLE (context->source_window),
+						     gdk_atom_intern (NAUTILUS_ICON_DND_XDNDDIRECTSAVE_TYPE, FALSE),
+						     gdk_atom_intern ("text/plain", FALSE), 8,
+						     GDK_PROP_MODE_REPLACE, (const guchar *) "", 0);
+            		} else if (drag_info->selection_data->format == 8 &&
+				   drag_info->selection_data->length == 1 &&
+				   drag_info->selection_data->data[0] == 'S' &&
+				   drag_info->direct_save_uri != NULL) {
+				GdkPoint p;
+				GFile *location;
+
+				location = g_file_new_for_uri (drag_info->direct_save_uri);
+
+				nautilus_file_changes_queue_file_added (location);
+				p.x = x; p.y = y;
+				nautilus_file_changes_queue_schedule_position_set (location,
+										   p,
+										   gdk_screen_get_number (gtk_widget_get_screen (widget)));
+				g_object_unref (location);
+				nautilus_file_changes_consume_changes (TRUE);
+            		}
+		        success = TRUE;
+			break;
 		}
 		gtk_drag_finish (context, success, FALSE, time);
 		
