@@ -30,7 +30,6 @@
 #include <libnautilus-private/nautilus-icon-canvas-item.h>
 #include <libnautilus-private/nautilus-icon-container.h>
 #include <libnautilus-private/nautilus-icon-dnd.h>
-#include <libnautilus-private/nautilus-icon-factory.h>
 
 /* An Icon. */
 
@@ -43,10 +42,17 @@ typedef struct {
 
 	/* X/Y coordinates. */
 	double x, y;
+
+	/*
+	 * In RTL mode x is RTL x position, we use saved_ltr_x for
+	 * keeping track of x value before it gets converted into
+	 * RTL value, this is used for saving the icon position 
+	 * to the nautilus metafile. 
+	 */
+	 double saved_ltr_x;
 	
 	/* Scale factor (stretches icon). */
-	/* ALEX TODO: only allow once scale */
-	double scale_x, scale_y;
+	double scale;
 
 	/* Whether this item is selected. */
 	eel_boolean_bit is_selected : 1;
@@ -76,7 +82,8 @@ typedef struct {
 	guint timer_id;
 
 	guint prev_x, prev_y;
-	ArtDRect prev_rect;
+	EelDRect prev_rect;
+	int last_adj_y;
 } NautilusIconRubberbandInfo;
 
 typedef enum {
@@ -104,6 +111,7 @@ enum {
 	LABEL_COLOR,
 	LABEL_COLOR_HIGHLIGHT,
 	LABEL_COLOR_ACTIVE,
+	LABEL_COLOR_PRELIGHT,
 	LABEL_INFO_COLOR,
 	LABEL_INFO_COLOR_HIGHLIGHT,
 	LABEL_INFO_COLOR_ACTIVE,
@@ -192,11 +200,13 @@ struct NautilusIconContainerDetails {
 	int font_size_table[NAUTILUS_ZOOM_LEVEL_LARGEST + 1];
 
 	/* pixbuf and color for label highlighting */
-	GdkPixbuf *highlight_frame;
 	guint32    highlight_color_rgba;
 	guint32    active_color_rgba;
 	guint32    normal_color_rgba;
-	
+	guint32    prelight_color_rgba;
+	guint32    prelight_icon_color_rgba;
+	guint32    normal_icon_color_rgba;
+
 	/* colors for text labels */
 	GdkGC   *label_gcs    [LAST_LABEL_COLOR];
 	GdkColor label_colors [LAST_LABEL_COLOR];
@@ -254,6 +264,8 @@ struct NautilusIconContainerDetails {
 	guint a11y_item_action_idle_handler;
 	GQueue* a11y_item_action_queue;
 
+	eel_boolean_bit is_reloading : 1;
+
 	/* interactive search */
 	gboolean disable_popdown;
 	gboolean imcontext_changed;
@@ -262,6 +274,9 @@ struct NautilusIconContainerDetails {
 	GtkWidget *search_entry;
 	guint search_entry_changed_id;
 	guint typeselect_flush_timeout;
+
+	/* Needed for dblclicking activation of partially shown icons, see bug #347423 */
+	gboolean icon_revealed;
 };
 
 /* Private functions shared by mutiple files. */
@@ -271,8 +286,7 @@ void          nautilus_icon_container_move_icon                   (NautilusIconC
 								   NautilusIcon          *icon,
 								   int                    x,
 								   int                    y,
-								   double                 scale_x,
-								   double                 scale_y,
+								   double                 scale,
 								   gboolean               raise,
 								   gboolean               snap,
 								   gboolean		  update_position);
@@ -297,6 +311,7 @@ void          nautilus_icon_container_update_scroll_region        (NautilusIconC
 GdkGC        *nautilus_icon_container_get_label_color_and_gc      (NautilusIconContainer *container,
 								   GdkColor             **color,
 								   gboolean               first_line,
-								   gboolean               needs_highlight);
+								   gboolean               needs_highlight,
+								   gboolean		     is_prelit);
 
 #endif /* NAUTILUS_ICON_CONTAINER_PRIVATE_H */
