@@ -114,6 +114,14 @@ static NautilusThumbnailInfo *currently_thumbnailing = NULL;
 
 static GnomeThumbnailFactory *thumbnail_factory = NULL;
 
+static const char *types [] = {
+	"image/x-bmp", "image/x-ico", "image/jpeg", "image/gif",
+	"image/png", "image/pnm", "image/ras", "image/tga",
+	"image/tiff", "image/wbmp", "image/bmp", "image/x-xbitmap",
+	"image/x-xpixmap"
+};
+static GHashTable *image_mime_types = NULL;
+
 static int thumbnail_icon_size = 0;
 
 static gboolean
@@ -130,6 +138,7 @@ get_file_mtime (const char *file_uri, time_t* mtime)
 		*mtime = g_file_info_get_attribute_uint64 (info, G_FILE_ATTRIBUTE_TIME_MODIFIED);
 		g_object_unref (info);
 	}
+	g_object_unref (file);
 	
 	return TRUE;
 }
@@ -727,6 +736,36 @@ thumbnail_thread_notify_file_changed (gpointer image_uri)
 	return FALSE;
 }
 
+static GHashTable *
+get_types_table (void)
+{
+	int i;
+
+	if (image_mime_types == NULL) {
+		image_mime_types = g_hash_table_new (g_str_hash, g_str_equal);
+		for (i = 0; i < G_N_ELEMENTS (types); i++) {
+			g_hash_table_insert (image_mime_types,
+					     (gpointer) types [i],
+					     GUINT_TO_POINTER (1));
+		}
+	}
+
+	return image_mime_types;
+}
+
+gboolean
+nautilus_thumbnail_is_mimetype_limited_by_size (const char *mime_type)
+{
+	GHashTable *image_mime_types;
+	
+	image_mime_types = get_types_table ();
+        if (g_hash_table_lookup (image_mime_types, mime_type)) {
+                return TRUE;
+	}
+
+        return FALSE;
+}
+
 gboolean
 nautilus_can_thumbnail (NautilusFile *file)
 {
@@ -751,6 +790,22 @@ nautilus_can_thumbnail (NautilusFile *file)
 	return res;
 }
 
+gboolean
+nautilus_has_valid_failed_thumbnail (NautilusFile *file)
+{
+	GnomeThumbnailFactory *factory;
+	char *uri;
+	gboolean res;
+
+	factory = get_thumbnail_factory ();
+
+	uri = nautilus_file_get_uri (file);
+	res = gnome_thumbnail_factory_has_valid_failed_thumbnail (factory, uri, file->details->mtime);
+	g_free (uri);
+
+	return res;
+}
+
 void
 nautilus_create_thumbnail (NautilusFile *file)
 {
