@@ -43,8 +43,8 @@
 #include <libnautilus-private/nautilus-global-preferences.h>
 #include <libnautilus-private/nautilus-sidebar-provider.h>
 #include <libnautilus-private/nautilus-module.h>
+#include <libnautilus-private/nautilus-signaller.h>
 
-#include "nautilus-signaller.h"
 #include "nautilus-history-sidebar.h"
 
 #define NAUTILUS_HISTORY_SIDEBAR_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), NAUTILUS_TYPE_HISTORY_SIDEBAR, NautilusHistorySidebarClass))
@@ -74,6 +74,8 @@ enum {
 static void  nautilus_history_sidebar_iface_init        (NautilusSidebarIface         *iface);
 static void  sidebar_provider_iface_init                (NautilusSidebarProviderIface *iface);
 static GType nautilus_history_sidebar_provider_get_type (void);
+static void  nautilus_history_sidebar_style_set	        (GtkWidget *widget,
+							 GtkStyle  *previous_style);
 
 G_DEFINE_TYPE_WITH_CODE (NautilusHistorySidebar, nautilus_history_sidebar, GTK_TYPE_SCROLLED_WINDOW,
 			 G_IMPLEMENT_INTERFACE (NAUTILUS_TYPE_SIDEBAR,
@@ -143,7 +145,7 @@ row_activated_callback (GtkTreeView *tree_view,
 	GtkTreeModel *model;
 	GtkTreeIter iter;
 	NautilusBookmark *bookmark;
-	char *uri;
+	GFile *location;
 	
 	sidebar = NAUTILUS_HISTORY_SIDEBAR (user_data);
 	model = gtk_tree_view_get_model (tree_view);
@@ -156,11 +158,11 @@ row_activated_callback (GtkTreeView *tree_view,
 		(model, &iter, HISTORY_SIDEBAR_COLUMN_BOOKMARK, &bookmark, -1);
 	
 	/* Navigate to the clicked location. */
-	uri = nautilus_bookmark_get_uri (NAUTILUS_BOOKMARK (bookmark));
+	location = nautilus_bookmark_get_location (NAUTILUS_BOOKMARK (bookmark));
 	nautilus_window_info_open_location
 		(sidebar->window, 
-		 uri, NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE, 0, NULL);
-	g_free (uri);
+		 location, NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE, 0, NULL);
+	g_object_unref (location);
 }
 
 static void
@@ -267,6 +269,8 @@ static void
 nautilus_history_sidebar_class_init (NautilusHistorySidebarClass *class)
 {
 	G_OBJECT_CLASS (class)->finalize = nautilus_history_sidebar_finalize;
+
+	GTK_WIDGET_CLASS (class)->style_set = nautilus_history_sidebar_style_set;
 }
 
 static const char *
@@ -318,6 +322,17 @@ nautilus_history_sidebar_set_parent_window (NautilusHistorySidebar *sidebar,
 	update_history (sidebar);
 }
 
+static void
+nautilus_history_sidebar_style_set (GtkWidget *widget,
+				   GtkStyle  *previous_style)
+{
+	NautilusHistorySidebar *sidebar;
+
+	sidebar = NAUTILUS_HISTORY_SIDEBAR (widget);
+
+	update_history (sidebar);
+}
+
 static NautilusSidebar *
 nautilus_history_sidebar_create (NautilusSidebarProvider *provider,
 				 NautilusWindowInfo *window)
